#ifndef __RT_INSTR__
#define __RT_INSTR__

#include <rtcollect.h>
#include <rtstring.h>

#include "robvars.h" // everyone include this!
#include "robbase.h"

using namespace lrt;

// abstract base class Instr is in robbase.h
namespace rt {

/*******/
/* DIE */
/*******/

class InstrDie : public Instr{

public:

  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrDie(const Globals& glob);
  ~InstrDie();
};

/********/
/* MOVE */
/********/

class InstrMove : public Instr{

public:

  virtual rint getNumCycles(Task* task);
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrMove(const Globals& glob);
  ~InstrMove();
};

/********/
/* TURN */
/********/

class InstrTurn : public Instr{

public:

  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrTurn(const Globals& glob, const Array<Op>& ops);

  ~InstrTurn();
};

/********/
/* SCAN */
/********/

class InstrScan : public Instr{

public:

  rint getNumCycles(Task* task);
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrScan(const Globals& glob, const Array<Op>& ops);
  ~InstrScan();
};

/***********/
/* FarScan */
/***********/

class InstrFarscan : public Instr{

public:

  rint getNumCycles(Task* task);
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrFarscan(const Globals& glob, const Array<Op>& ops);
  ~InstrFarscan();
};

/*******/
/* SET */
/*******/

class InstrSet : public Instr{

public:

  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrSet(const Globals& glob, const Array<Op>& ops);
  ~InstrSet();
};

/*******/
/* MIN */
/*******/

class InstrMin : public Instr{

public:

  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrMin(const Globals& glob, const Array<Op>& ops);
  ~InstrMin();
};


/*******/
/* MAX */
/*******/

class InstrMax : public Instr{

public:

  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrMax(const Globals& glob, const Array<Op>& ops);
  ~InstrMax();
};


/*******/
/* ADD */
/*******/

class InstrAdd : public Instr{

public:

  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrAdd(const Globals& glob, const Array<Op>& ops);
  ~InstrAdd();
};


/*******/
/* SUB */
/*******/

class InstrSub : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();
  InstrSub(const Globals& glob, const Array<Op>& ops);
  ~InstrSub();
};


/*******/
/* MUL */
/*******/

class InstrMul : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrMul(const Globals& glob, const Array<Op>& ops);
  ~InstrMul();
};


/*******/
/* DIV */
/*******/

class InstrDiv : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrDiv(const Globals& glob, const Array<Op>& ops);
  ~InstrDiv();
};


/*******/
/* MOD */
/*******/

class InstrMod : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrMod(const Globals& glob, const Array<Op>& ops);
  ~InstrMod();
};


/*********/
/* TRANS */
/*********/

class InstrTrans : public Instr{

public:
  virtual rint getNumCycles(Task* task);
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrTrans(const Globals& glob, const Array<Op>& ops);
  ~InstrTrans();
};

/**********/
/* RTRANS */
/**********/

class InstrRtrans : public Instr{

public:
  virtual rint getNumCycles(Task* task);
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrRtrans(const Globals& glob, const Array<Op>& ops);
  ~InstrRtrans();
};

/*********/
/* AJUMP */
/*********/

class InstrAjump : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrAjump(const Globals& glob, const Array<Op>& ops);
  ~InstrAjump();
};


/********/
/* JUMP */
/********/

class InstrJump : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrJump(const Globals& glob, const Array<Op>& ops);
  ~InstrJump();
};


/*********/
/* BJUMP */
/*********/

class InstrBjump : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrBjump(const Globals& glob, const Array<Op>& ops);
  ~InstrBjump();
};


/*********/
/* SLEEP */
/*********/

class InstrSleep : public Instr{

public:
  virtual rint getNumCycles(Task* task);
  virtual ExecReturnType exec(Task* task);
  virtual TimingMode getTimingMode();

  String getName();

  InstrSleep(const Globals& glob, const Array<Op>& ops);
  ~InstrSleep();
};


/********/
/* COMP */
/********/

class InstrComp : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrComp(const Globals& glob, const Array<Op>& ops);
  ~InstrComp();
};


/*********/
/* NCOMP */
/*********/

class InstrNcomp : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrNcomp(const Globals& glob, const Array<Op>& ops);
  ~InstrNcomp();
};


/*********/
/* LCOMP */
/*********/

class InstrLcomp : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrLcomp(const Globals& glob, const Array<Op>& ops);
  ~InstrLcomp();
};


/*********/
/* GCOMP */
/*********/

class InstrGcomp : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrGcomp(const Globals& glob, const Array<Op>& ops);
  ~InstrGcomp();
};


/**********/
/* CREATE */
/**********/

class InstrCreate : public Instr{

public:
  virtual rint getNumCycles(Task* task);
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrCreate(const Globals& glob, const Array<Op>& ops);
  ~InstrCreate();
};

/**********/
/* RANDOM */
/**********/

class InstrRandom : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrRandom(const Globals& glob, const Array<Op>& ops);
  ~InstrRandom();
};

////////////////////////////// Multitasking /////////////////////////

/*********/
/* BREAK */
/*********/

class InstrBreak : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrBreak(const Globals& glob);
  ~InstrBreak();
};

/*********/
/* INIT  */
/*********/

class InstrInit : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrInit(const Globals& glob, const Array<Op>& ops);
  ~InstrInit();
};

/*********/
/* SEIZE */
/*********/

class InstrSeize : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrSeize(const Globals& glob);
  ~InstrSeize();
};

/**********/
/* RESUME */
/**********/

class InstrResume : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrResume(const Globals& glob);
  ~InstrResume();
};

/********/
/* QUIT */
/********/

class InstrQuit : public Instr{

public:
  virtual ExecReturnType exec(Task* task);

  String getName();

  InstrQuit(const Globals& glob);
  ~InstrQuit();
};





} // namespace


#endif

